#ifndef HYPER_MODEL_MODULE_ABILITY_TEST_HH
#define HYPER_MODEL_MODULE_ABILITY_TEST_HH


#include <model/ability.hh>
#include <model/future.hh>
#include <model/proxy.hh>
#include <network/log_level.hh>
#include <network/msg_constraint.hh>

#include <boost/thread/thread.hpp>

namespace hyper {
	namespace model {
		struct ability_test : public ability {
			std::string target;
			boost::thread thr;
			
			ability_test(const std::string& name_) : 
				ability(name_ +  "_test", DEBUG_ALL), target(name_),
				thr(boost::bind(&ability::run, this))
			{
				start();
			}

			private:
			template <typename T>
			void handle_get_value(const boost::system::error_code&e,
					remote_proxy* proxy,
					future_value<T> future)
			{
				delete proxy;
				if (e || !future.get_raw()) {
					std::cerr << "Failed to get the value of " << future.name() << std::endl;
				} else {
					std::cerr << "Successful get the value of " << future.name() << std::endl;
				}
				future.signal_ready();
			}

			void handle_get_value(const boost::system::error_code&e,
								  remote_proxy* proxy,
								  const std::string& value);

			void handle_send_constraint(const boost::system::error_code& e,
										network::identifier id,
										future_value<bool>, 
									    network::request_constraint* msg,
										network::request_constraint_answer* ans);

			protected:
			template <typename T>
			future_value<T> get_value(const std::string& value)
			{
				remote_proxy* proxy = 
					new remote_proxy (*this);
			
				future_value<T> res(value);

				void (ability_test::*f) (const boost::system::error_code&,
										 remote_proxy*,
										 future_value<T>) =
					&ability_test::template handle_get_value<T>;

				proxy->async_get(target, res.name(), res.get_raw(),
						boost::bind(f, this,
									boost::asio::placeholders::error,
									proxy, res));

				return res;
			}

			public:
			future_value<bool> send_constraint(const std::string& constraint, bool repeat);
			void abort(const std::string& reason);
		};
	}
}

#endif /* HYPER_MODEL_MODULE_ABILITY_TEST_HH */
